/*
    $Id: file_output.h 179 2010-12-16 21:09:18Z matwey $
    Copyright (C) 2009  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _FILE_OUTPUT_H
#define _FILE_OUTPUT_H

#include <iosfwd>
#include <string>

#include <boost/format.hpp>
#include <boost/io/ios_state.hpp>
#include <boost/numeric/ublas/symmetric.hpp>
#include <boost/numeric/ublas/vector.hpp>

#include <boost/date_time/posix_time/posix_time.hpp>

#include "io.h"

using namespace boost::numeric;

class file_output {
public:
	typedef size_t size_type;

	typedef boost::posix_time::ptime time_type;
	typedef object                   object_type;
private:
	std::ostream& m_stm;
public:
	file_output( std::ostream& stm );
	~file_output();

	void comment( const std::string& comment );
	void normal( const time_type& time, double mz );
	void background( const time_type& time );
	void atmosphere( const time_type& time, double free_seeing, double free_seeing_err, double dimm_seeing, double dimm_seeing_err, double dimm_turbulence, double dimm_turbulence_err, double isoplanatic_angle, double isoplanatic_angle_err, double altitude_eff, double altitude_eff_err, double time_constant, double time_constant_err, double moment_0, double moment_0_err, double moment_2, double moment_2_err );
	void profile( const time_type& time, const std::string& method, double cn2seeing, double chi2, const ublas::vector<double>& cn2, const ublas::vector<double>& ecn2, const ublas::vector<double>& grid );
	void residual( const time_type& time, const std::string& method, const ublas::vector<double>& o_csi );
	void fluxes( const time_type& time, const ublas::vector<double>& result_flux );
	void fluxes( const time_type& time, const ublas::vector<double>& result_flux, const ublas::vector<double>& result_flux_err );
	void indices( const time_type& time, const ublas::symmetric_matrix<double>& result_indx, const ublas::symmetric_matrix<double>& result_indx_err, const ublas::vector<double>& result_desi, const ublas::vector<double>& result_desi_err );
	void parameter( const time_type& time, const std::string& key, const std::string& value );
	void objectinfo( const time_type& time, const object_type& obj );
	void wind( const time_type& time, const std::string& mode, double chi2, double tau0, const ublas::vector<double>& profile, const ublas::vector<double>& eps, const ublas::vector<double>& grid );

	std::ostream& operator()() { return m_stm; }
};

#endif // _FILE_OUTPUT_H
