/*
    $Id: merge_join.h 171 2010-11-07 18:14:04Z matwey $
    Copyright (C) 2010  Sternberg Astronomical Institute, MSU

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _MERGE_JOIN_H
#define _MERGE_JOIN_H

#include <utility>

namespace utils{
namespace algo{

template<class Iterator1, class Iterator2, class OutputIterator, class BinaryOperator>
void merge_join( Iterator1 begin1, Iterator1 end1,
		 Iterator2 begin2, Iterator2 end2, OutputIterator result, BinaryOperator less ) {
	using namespace std;
	
	Iterator1 it1 = begin1;
	Iterator2 it2 = begin2;

	while( it1 != end1 && it2 != end2 ) {
		if( less( *it1, *it2 ) ) ++it1;
		else if( less( *it2, *it1 ) ) ++it2;
		else *result++ = make_pair( *it1++, *it2++ );
	}
}

template<class Iterator1, class Iterator2, class OutputIterator, class UnaryOperator1, class UnaryOperator2>
void merge_join( Iterator1 begin1, Iterator1 end1,
		 Iterator2 begin2, Iterator2 end2, OutputIterator result, UnaryOperator1 key1, UnaryOperator2 key2 ) {
	typedef typename UnaryOperator1::result_type key1_value_type;
	typedef typename UnaryOperator2::result_type key2_value_type;
	
	using namespace std;

	Iterator1 it1 = begin1;
	Iterator2 it2 = begin2;

	while( it1 != end1 && it2 != end2 ) {
		const key1_value_type& k1 = key1(*it1);
		const key2_value_type& k2 = key2(*it2);

		if( k1 < k2 ) ++it1;
		else if( k2 < k1 ) ++it2;
		else *result++ = make_pair( *it1++, *it2++ );
	}
}

} // algo
} // utlis

#endif // _MERGE_JOIN_H
